package com.wombat.aliyun_oss

import android.app.Activity
import android.app.Activity.RESULT_OK
import android.content.Intent
import android.os.Handler
import android.os.Looper
import android.util.Log
import androidx.annotation.NonNull
import com.alibaba.sdk.android.oss.ClientConfiguration
import com.alibaba.sdk.android.oss.OSSClient
import com.alibaba.sdk.android.oss.common.auth.OSSPlainTextAKSKCredentialProvider
import com.alibaba.sdk.android.oss.common.auth.OSSStsTokenCredentialProvider
import com.alibaba.sdk.android.oss.common.utils.BinaryUtil
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import kotlin.concurrent.thread

/** AliyunOSSPlugin */
class AliyunOSSPlugin : FlutterPlugin, MethodCallHandler, ActivityAware,
    PluginRegistry.ActivityResultListener, EventChannel.StreamHandler {
    private lateinit var methodChannel: MethodChannel
    private lateinit var eventChannel: EventChannel
    private var activity: Activity? = null
    private var ossService: OSSService? = null
    private var activityPluginBinding: ActivityPluginBinding? = null
    private var bucketName: String = ""

    private val RESULT_LOAD_IMAGE = 101

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel =
            MethodChannel(flutterPluginBinding.binaryMessenger, "com.wombat/aliyun_oss_plugin")

        methodChannel.setMethodCallHandler(this)

        eventChannel = EventChannel(
            flutterPluginBinding.binaryMessenger,
            "com.wombat/aliyun_oss_stream_plugin"
        )
        eventChannel.setStreamHandler(this)

    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
        binding.addActivityResultListener(this)
        activityPluginBinding = binding
    }

    override fun onDetachedFromActivityForConfigChanges() {
        TODO("Not yet implemented")
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        TODO("Not yet implemented")
    }

    override fun onDetachedFromActivity() {
        activity = null
        activityPluginBinding?.removeActivityResultListener(this)
        activityPluginBinding = null
    }

    private fun uploadProgress(
        bytesSent: Long,
        totalBytesSent: Long,
        totalBytesExpectedToSend: Long,
        finish: Boolean
    ) {
        activity?.runOnUiThread {
            methodChannel.invokeMethod(
                "upload_progress",
                hashMapOf(
                    "bytesSent" to bytesSent,
                    "totalBytesSent" to totalBytesSent,
                    "totalBytesExpectedToSend" to totalBytesExpectedToSend,
                    "finish" to finish
                )
            )
        }
    }

    private fun uploadFail(error: String) {
        activity?.runOnUiThread {
            methodChannel.invokeMethod(
                "upload_fail",
                hashMapOf("error" to error)
            )
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        // 用于处理选择文件的结果监听
        if (resultCode != RESULT_OK) {
            return false
        }
        if (requestCode == RESULT_LOAD_IMAGE && null != data) {
            val filePath = RealPathUtil.getRealPath(activity!!, data.data!!)
            if (filePath != null) {
                val extension =
                    if (filePath.contains(".")) {
                        filePath.substring(filePath.lastIndexOf("."))
                    } else {
                        filePath
                    }
                Log.d("开始上传文件", extension)

                thread {
                    val start = System.currentTimeMillis()
                    val md5 = BinaryUtil.calculateMd5(filePath)
                    Log.d("計算md5耗時","${System.currentTimeMillis() - start}")
                    Log.d("上传名称objectKey", "${md5}${extension}")
                    ossService?.asyncMultipartUpload(
                        bucketName,
                        "${md5}${extension}",
                        data.data!!,
                        this::uploadProgress,
                        this::uploadFail
                    )
                }

                return true
            }

        }

        return false
    }


    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "init" -> {
                initOSSClient(call.arguments as HashMap<String, Any>)
                result.success("初始化成功")
            }

            "putObject" -> {
                putObject(call.argument<String>("bucketName") as String)
            }
            else -> result.notImplemented()
        }
    }

    override fun onListen(arguments: Any?, events: EventSink?) {
        startListening(arguments, events)
    }

    override fun onCancel(arguments: Any?) {
        cancelListening(arguments)
    }

    private var listeners = HashMap<Any?, Runnable>()

    private fun startListening(listener: Any?, emitter: EventSink?) {
        val handler = Handler(Looper.getMainLooper())
        listeners[listener] = object : Runnable {
            override fun run() {
                if (listeners.containsKey(listener)) {
                    // Send some value to callback
                    emitter?.success("Hello listener! " + System.currentTimeMillis() / 1000)
                    handler.postDelayed(this, 1000)
                }
            }
        }
        // Run task
        handler.postDelayed(listeners[listener]!!, 1000)
    }

    private fun cancelListening(listener: Any?) {
        listeners.remove(listener);
    }

    /**
     * 初始化 OSS 客户端
     */
    private fun initOSSClient(params: HashMap<String, Any>) {
        Log.d("参数", "$params")
        val endpoint = params["endpoint"] as String
        val accessKeyId = params["accessKeyId"] as String
        val secretKeyId = params["secretKeyId"] as String
        val securityToken = params["securityToken"] as String?


        val conf = ClientConfiguration().apply {
            connectionTimeout = 15 * 1000 // 连接超时，默认15秒

            socketTimeout = 15 * 1000 // socket超时，默认15秒

            maxConcurrentRequest = 5 // 最大并发请求数，默认5个

            maxErrorRetry = 2 // 失败后最大重试次数，默认2次
        }


        val credential = if (securityToken == null) {
            OSSPlainTextAKSKCredentialProvider(accessKeyId, secretKeyId)
        } else {
            OSSStsTokenCredentialProvider(accessKeyId, secretKeyId, securityToken)
        }
        ossService = OSSService.getInstance(OSSClient(activity!!, endpoint, credential, conf))
    }

    /**
     * 上传文件，回传上传进度，处理分片/断点续传
     */
    private fun putObject(bucketName: String) {
        this.bucketName = bucketName
        activity?.run {
//            val intent = Intent(
//                Intent.ACTION_PICK,
//                MediaStore.Images.Media.EXTERNAL_CONTENT_URI
//            )
            val intent = Intent(
                Intent.ACTION_GET_CONTENT
            )
            intent.type = "*/*"
            startActivityForResult(intent, RESULT_LOAD_IMAGE)
        }
    }
}
