package com.wombat.aliyun_oss

import android.net.Uri
import android.util.Log
import com.alibaba.sdk.android.oss.ClientException
import com.alibaba.sdk.android.oss.OSS
import com.alibaba.sdk.android.oss.ServiceException
import com.alibaba.sdk.android.oss.callback.OSSCompletedCallback
import com.alibaba.sdk.android.oss.callback.OSSProgressCallback
import com.alibaba.sdk.android.oss.common.OSSLog
import com.alibaba.sdk.android.oss.internal.OSSAsyncTask
import com.alibaba.sdk.android.oss.model.*
import java.io.File


class OSSService private constructor(private val oss: OSS) {
    companion object {
        @Volatile
        private var INSTANCE: OSSService? = null

        fun getInstance(oss: OSS): OSSService {
            if (INSTANCE == null) {
                synchronized(this) {
                    if (INSTANCE == null) {
                        INSTANCE = OSSService(oss)
                    }
                }
            }
            return INSTANCE!!
        }
    }

    fun asyncPutObject(
        bucketName: String,
        objectKey: String,
        localFile: String,
        progressCallback: (bytesSent: Long, totalBytesSent: Long, totalBytesExpectedToSend: Long, finish: Boolean) -> Unit,
        failCallback: (error: String) -> Unit
    ) {
        val uploadStart = System.currentTimeMillis()
        OSSLog.logDebug("upload start")
        if (objectKey.isEmpty()) {
            Log.w("AsyncPutImage", "ObjectNull")
            return
        }
        val file = File(localFile)
        if (!file.exists()) {
            Log.w("AsyncPutImage", "FileNotExist")
            Log.w("LocalFile", localFile)
            return
        }

        // 构造上传请求
        OSSLog.logDebug("create PutObjectRequest ")
        val put = PutObjectRequest(bucketName, objectKey, localFile)

        put.crC64 = OSSRequest.CRC64Config.YES
        // 异步上传时可以设置进度回调
        put.progressCallback =
            OSSProgressCallback { _, currentSize, totalSize ->
                progressCallback(-1, currentSize, totalSize, false)
            }

        val task: OSSAsyncTask<*> = oss.asyncPutObject(
            put,
            object : OSSCompletedCallback<PutObjectRequest, PutObjectResult> {
                override fun onSuccess(request: PutObjectRequest, result: PutObjectResult) {
                    Log.d("PutObject", "UploadSuccess")
                    Log.d("ETag", result.eTag)
                    Log.d("RequestId", result.requestId)
                    val uploadEnd = System.currentTimeMillis()
                    OSSLog.logDebug("上传耗时: " + (uploadEnd - uploadStart) / 1000f)
                    Log.d("上传耗时", "${(uploadEnd - uploadStart) / 1000f}")
                    progressCallback.invoke(-1, -1, -1, true)
                }

                override fun onFailure(
                    request: PutObjectRequest,
                    clientException: ClientException?,
                    serviceException: ServiceException?
                ) {
                    var info = ""
                    // 请求异常
                    if (clientException != null) {
                        // 本地异常如网络异常等
                        clientException.printStackTrace()
                        info = clientException.toString()
                    }
                    if (serviceException != null) {
                        // 服务异常
                        Log.e("ErrorCode", serviceException.errorCode)
                        Log.e("RequestId", serviceException.requestId)
                        Log.e("HostId", serviceException.hostId)
                        Log.e("RawMessage", serviceException.rawMessage)
                        info = serviceException.toString()
                    }
                    Log.e("错误", "$request")
                    failCallback.invoke(info)
                }
            })

    }

    fun asyncMultipartUpload(
        bucketName: String,
        objectKey: String,
        fileUri: Uri,
        progressCallback: (bytesSent: Long, totalBytesSent: Long, totalBytesExpectedToSend: Long, finish: Boolean) -> Unit,
        failCallback: (error: String) -> Unit
    ) {
        val uploadStart = System.currentTimeMillis()
        OSSLog.logDebug("upload start")
        if (objectKey.isEmpty()) {
            Log.w("AsyncUploadRequest", "ObjectNull")
            return
        }
        // 构造上传请求
        OSSLog.logDebug("create MultipartUploadRequest ")

        val rq =
            OSSServiceExtension.createMultipartUploadRequest(bucketName, objectKey, fileUri)

        rq.progressCallback =
            OSSProgressCallback { _, currentSize, totalSize ->
                progressCallback(-1, currentSize, totalSize, false)
            }

        val task = oss.asyncMultipartUpload(
            rq,
            object :
                OSSCompletedCallback<MultipartUploadRequest<*>, CompleteMultipartUploadResult> {
                override fun onSuccess(
                    request: MultipartUploadRequest<*>,
                    result: CompleteMultipartUploadResult
                ) {
                    Log.d("MultipartUpload", "UploadSuccess")
                    Log.d("ETag", result.eTag)
                    Log.d("RequestId", result.requestId)
                    val uploadEnd = System.currentTimeMillis()
                    OSSLog.logDebug("上传耗时: " + (uploadEnd - uploadStart) / 1000f)
                    Log.d("上传耗时", "${(uploadEnd - uploadStart) / 1000f}")
                    progressCallback.invoke(-1, -1, -1, true)
                }

                override fun onFailure(
                    request: MultipartUploadRequest<*>,
                    clientException: ClientException?,
                    serviceException: ServiceException?
                ) {
                    var info = ""
                    // 请求异常
                    if (clientException != null) {
                        // 本地异常如网络异常等
                        clientException.printStackTrace()
                        info = clientException.toString()
                    }
                    if (serviceException != null) {
                        // 服务异常
                        Log.e("ErrorCode", serviceException.errorCode)
                        Log.e("RequestId", serviceException.requestId)
                        Log.e("HostId", serviceException.hostId)
                        Log.e("RawMessage", serviceException.rawMessage)
                        info = serviceException.toString()
                    }
                    Log.e("错误", "$request")
                    failCallback.invoke(info)
                }
            })
    }

}